@@ -12,9 +12,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -27,7 +27,7 @@
 static bool
 connectmgr_get_protocol_info(ActionEvent *event)
 {
-    upnp_add_response(event, "SOURCE", "http-get:*:audio/mpeg:*"); /* XXX format */
+    upnp_add_response(event, "SOURCE", "http-get:*:audio/mpeg:*"); /* XXX format */ /* FIXME only one type?! */
     upnp_add_response(event, "SINK", "");
 
     return event->status;
