@@ -12,9 +12,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -107,7 +107,7 @@ void
 begin_result(StrBuf *result)
 {
     strbuf_append(result, 
-            "<DIDL-Lite xmlns=\"urn:schemas-upnp-org:metadata-1-0/DIDL-Lite\""
+            "<DIDL-Lite xmlns=\"urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/\""
             " xmlns:dc=\"http://purl.org/dc/elements/1.1/\""
             " xmlns:upnp=\"urn:schemas-upnp-org:metadata-1-0/upnp/\">");
 }
@@ -148,8 +148,20 @@ get_entry_property(Entry *entry, char *property)
     return NULL;
 }
 
+static bool
+filter_includes(const char *filter, const char *field)
+{
+    if (strcmp(filter, "*") == 0)
+        return true;
+
+    if (string_in_csv(filter, field))
+	return true;
+
+    return false;
+}
+
 static void
-append_entry(StrBuf *result, Entry *entry)
+append_entry(StrBuf *result, Entry *entry, const char *filter)
 {
     EntryDetail *detail;
     char *value;
@@ -166,28 +178,43 @@ append_entry(StrBuf *result, Entry *entry)
 		"<item id=\"%" PRIi32 "\" parentID=\"%" PRIi32 "\" restricted=\"true\">",
 		entry->id,
 		entry->parent);
-	strbuf_appendf(result,
-		"<res protocolInfo=\"http-get:*:audio/mpeg:*\">http://%s:%d/audio/%d</res>",
-		UpnpGetServerIpAddress(),
-		UpnpGetServerPort(),
-		entry->id);
+        if (filter_includes(filter, "res") || filter_includes(filter, "@protocolInfo") || filter_includes(filter, "@size")) {
+            strbuf_append(result, "<res");
+            if (filter_includes(filter, "res") || filter_includes(filter, "@protocolInfo"))
+                strbuf_appendf(result, " protocolInfo=\"http-get:*:%s:*\"", entry->mime_type);
+	    /*if (entry->size != 0)*/
+	    if (filter_includes(filter, "res") || filter_includes(filter, "@size"))
+	        strbuf_appendf(result, " size=\"%" PRIu32 "\"", entry->size);
+            strbuf_appendf(result, ">http://%s:%d/files/%d</res>",
+		    UpnpGetServerIpAddress(),
+		    UpnpGetServerPort(),
+		    entry->id);
+        }
     }
 
-    strbuf_appendf(result, "<class>%s</class>", get_entry_property(entry, "upnp:class"));
+    /* upnp:class is required and cannot be filtered out. */
+    strbuf_appendf(result, "<upnp:class>%s</upnp:class>", get_entry_property(entry, "upnp:class"));
 
-    /* XXX: dc:creator instead or in combination with upnp:artist? */
+    /* dc:title is required and cannot be filtered out. */
     value = get_entry_property(entry, "dc:title");
     if (value != NULL)
         append_escaped_xml(result, "dc:title", value);
-    value = get_entry_property(entry, "upnp:artist");
-    if (value != NULL)
-	append_escaped_xml(result, "upnp:artist", value);
-    value = get_entry_property(entry, "upnp:album");
-    if (value != NULL)
-	append_escaped_xml(result, "upnp:album", value);
-    value = get_entry_property(entry, "upnp:genre");
-    if (value != NULL)
-	append_escaped_xml(result, "upnp:genre", value);
+    /* XXX: dc:creator instead or in combination with upnp:artist? */
+    if (filter_includes(filter, "upnp:artist")) {
+        value = get_entry_property(entry, "upnp:artist");
+        if (value != NULL)
+	    append_escaped_xml(result, "upnp:artist", value);
+    }
+    if (filter_includes(filter, "upnp:album")) {
+        value = get_entry_property(entry, "upnp:album");
+        if (value != NULL)
+	    append_escaped_xml(result, "upnp:album", value);
+    }
+    if (filter_includes(filter, "upnp:genre")) {
+        value = get_entry_property(entry, "upnp:genre");
+        if (value != NULL)
+	    append_escaped_xml(result, "upnp:genre", value);
+    }
 
     if (detail != NULL) {
 	strbuf_append(result, "</container>");
@@ -364,7 +391,7 @@ match_search_criteria(SearchCriteria *criteria, Entry *entry)
 static uint32_t
 search_container(SearchCriteria *criteria, Entry *entry,
 		 uint32_t *skip, uint32_t max_count,
-		 StrBuf *result)
+		 const char *filter, StrBuf *result)
 {
     EntryDetail *detail;
     uint32_t c;
@@ -381,7 +408,8 @@ search_container(SearchCriteria *criteria, Entry *entry,
 	    if (*skip > 0) {
 		(*skip)--;
 	    } else {
-		append_entry(result, child);
+	        /*if (result != NULL)*/
+                append_entry(result, child, filter);
 		match_count++;
 		if (match_count > max_count)
 		    return match_count;
@@ -389,8 +417,12 @@ search_container(SearchCriteria *criteria, Entry *entry,
 	}
 
 	if (has_entry_detail(child, DETAIL_CHILDREN)) {
-	    match_count += search_container(criteria, child, skip,
-		    max_count-match_count, result);
+	    /* Even if max_count is UINT32_MAX (for RequestedCount=0)
+	     * we allow it to be decreased. This is because we never
+	     * can return more than UINT32_MAX results anyway
+	     * (the return value would overflow).
+	     */
+	    match_count += search_container(criteria, child, skip, max_count-match_count, filter, result);
 	    if (match_count > max_count)
 		return match_count;
 	}
@@ -409,7 +441,7 @@ contentdir_search(ActionEvent *event)
     char *search_criteria;
     char *sort_criteria;
     char *filter;
-    const char *error;
+    const char *error = NULL;
     Entry *entry;
     EntryDetail *detail;
     SearchCriteria *criteria;
@@ -425,17 +457,10 @@ contentdir_search(ActionEvent *event)
     if (!event->status)
 	return false;
 
-    /* If no results are to be returned, the only plausible purpose of the
-     * action must be to query how many results there are through
-     * TotalMatches. Unfortunately we do not support TotalMatches, so we
-     * inform this by an error.
-     */
-    if (count == 0) {
-        upnp_set_error(event, UPNP_CONTENTDIR_E_CANNOT_PROCESS,
-            _("No results asked for"));
-	return false;
-    }
-    
+    /* A RequestedCount of 0 means that all results are to be returned. */
+    if (count == 0)
+        count = UINT32_MAX;
+
     /* Check ContainerID argument */
     entry = get_entry_by_id(id);
     if (entry == NULL) {
@@ -472,7 +497,7 @@ contentdir_search(ActionEvent *event)
     /* Do the actual searching */
     result = strbuf_new();
     begin_result(result);
-    match_count = search_container(criteria, entry, &index, count, result);
+    match_count = search_container(criteria, entry, &index, count, filter, result);
     end_result(result);
 
     /* Make response. */
@@ -510,6 +535,10 @@ contentdir_browse(ActionEvent *event)
     if (!event->status)
 	return false;
 
+    /* A RequestedCount of 0 means that all results are to be returned. */
+    if (count == 0)
+        count = UINT32_MAX;
+
     /* Check arguments */
     if (strcmp(sort_criteria, "") != 0) {
         upnp_set_error(event, UPNP_CONTENTDIR_E_BAD_SORT_CRITERIA,
@@ -542,7 +571,7 @@ contentdir_browse(ActionEvent *event)
 
 	result = strbuf_new();
 	begin_result(result);
-	append_entry(result, entry);
+	append_entry(result, entry, filter);
 	end_result(result);
         upnp_add_response(event, "Result", strbuf_buffer(result));
         upnp_add_response(event, "NumberReturned", "1");
@@ -572,10 +601,12 @@ contentdir_browse(ActionEvent *event)
 
 	result = strbuf_new();
 	begin_result(result);
-	end_index = MIN(index+count, detail->data.children.count);
+	end_index = detail->data.children.count;
+	if (count != UINT32_MAX) /* hmm.. should figure out a better way to detect if index+count overflows (>UINT32_MAX) */
+	    end_index = MIN(index+count, end_index);
         for (c = index; c < end_index; c++) {
 	    Entry *child = get_entry_by_id(detail->data.children.list[c]);
-	    append_entry(result, child);
+	    append_entry(result, child, filter);
 	    result_count++;
         }
 	end_result(result);
