@@ -12,9 +12,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -84,8 +84,10 @@ struct _Entry {
     int32_t id;
     int32_t parent;
     char *filename;
-    uint32_t detail_flags;
+    const char *mime_type;
+    uint32_t size;
     EntryDetail *details;
+    /*uint32_t detail_flags; 		unused for now */
 };
 
 struct _SearchExpr {
@@ -132,12 +134,16 @@ void init_logging(const char *logfilename, const char *timestamp_format);
 void finish_logging(bool success);
 
 /* main.c */
+bool string_in_csv(const char *csv, const char *value);
 
 /* interface.h */
 char *get_ip_by_spec(char *spec);
 
 /* metadata.c */
+extern char *file_types;
+#ifdef HAVE_ID3LIB
 extern bool id3_enabled;
+#endif
 bool init_metadata(char **pathv, int pathc);
 void finish_metadata(void);
 Entry *get_entry_by_id(uint32_t id);
@@ -167,6 +173,7 @@ bool has_entry_detail(Entry *entry, EntryDetailType type);
 #define UPNP_CONTENTDIR_E_NO_DST_ACCESS		719
 #define UPNP_CONTENTDIR_E_CANNOT_PROCESS	720
 extern char *friendly_name;
+extern int ssdp_expire_time;
 void init_upnp(const char *listenip, uint16_t listenport);
 void finish_upnp(void);
 void upnp_set_error(ActionEvent *event, int error_code, const char *format, ...);
