@@ -12,9 +12,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -36,6 +36,9 @@
 #define N_(s) gettext_noop(s)
 #include "xalloc.h"		/* Gnulib */
 #include "version-etc.h"	/* Gnulib */
+#include "progname.h"		/* Gnulib */
+#include "sig2str.h"		/* Gnulib */
+#include "tempfailure.h"
 #include "intutil.h"
 #include "gmediaserver.h"
 
@@ -46,6 +49,9 @@ enum {
     OPT_PORT,
     OPT_NOTIMESTAMP,
     OPT_TIMESTAMP,
+    OPT_EXPIRE_TIME,
+    OPT_FILE_TYPES,
+    OPT_PROFILE,
     OPT_HELP,
     OPT_VERSION,
 };
@@ -58,6 +64,7 @@ static struct option long_options[] = {
 #endif
     { "friendly-name", required_argument, NULL, OPT_FRIENDLY_NAME },
     { "pid-file", required_argument, NULL, OPT_PIDFILE },
+    { "profile", required_argument, NULL, OPT_PROFILE, },
     { "port", required_argument, NULL, 'p' },
     { "output", required_argument, NULL, 'o' },
     { "background", no_argument, NULL, 'b' },
@@ -65,6 +72,8 @@ static struct option long_options[] = {
     { "timestamp", optional_argument, NULL, OPT_TIMESTAMP },
     { "interface", required_argument, NULL, 'i' },
     { "verbose", optional_argument, NULL, 'v' },
+    { "expire-time", required_argument, NULL, OPT_EXPIRE_TIME },
+    { "file-types", required_argument, NULL, OPT_FILE_TYPES },
     { "help", no_argument, NULL, OPT_HELP },
     { "version", no_argument, NULL, OPT_VERSION },
     { NULL, 0, NULL, 0 }
@@ -72,6 +81,23 @@ static struct option long_options[] = {
 
 const char version_etc_copyright[] = "Copyright (C) 2005 Oskar Liljeblad.";
 
+/* XXX: move this to strutil.c or something */
+bool
+string_in_csv(const char *csv, const char *value)
+{
+    const char *p0;
+    const char *p1;
+
+    for (p0 = csv; (p1 = strchr(p0, ',')) != NULL; p0 = p1+1) {
+        if (strncmp(p0, value, p1-p0) == 0)
+            return true;
+    }
+    if (strcmp(p0, value) == 0)
+        return true;
+
+    return false;
+}
+
 static void
 dummy_signal_handler(int signal)
 {
@@ -92,7 +118,7 @@ write_pid_file(const char *pidfilename, pid_t pid, bool background)
 	    say(2, _("Writing pid to pid file `%s'\n"), pidfilename);
 	if ((pidfh = fopen(pidfilename, "w")) == NULL)
 	    die(_("Cannot open pid file `%s' for writing - %s\n"), pidfilename, errstr);
-	fprintf(pidfh, "%d", getpid());
+	fprintf(pidfh, "%d", pid);
 	if (fclose(pidfh) != 0)
 	    die(_("Cannot close pid file `%s' - %s\n"), pidfilename, errstr);
     }
@@ -110,6 +136,9 @@ main(int argc, char **argv)
     bool timestamp_specified = false;
     char *logfilename = NULL;
     char *timestamp_format = NULL;
+    uint32_t expire_time;
+
+    set_program_name(argv[0]);
 
     for (;;) {
     	int c;
@@ -136,6 +165,10 @@ main(int argc, char **argv)
 		die(_("friendly name cannot be empty\n"));
 	    friendly_name = xstrdup(optarg);
 	    break;
+	case OPT_FILE_TYPES:
+	    file_types = optarg;
+	    /* FIXME: check valid */
+	    break;
 	case OPT_PIDFILE:
 	    pidfilename = optarg;
 	    break;
@@ -143,6 +176,12 @@ main(int argc, char **argv)
 	    if (!parse_uint16(optarg, &listenport))
 		die(_("invalid port number `%s'\n"), optarg);
 	    break;
+        case OPT_EXPIRE_TIME:
+            if (!parse_uint32(optarg, &expire_time) || expire_time > INT_MAX)
+		die(_("invalid expire time `%s'\n"), optarg);
+            ssdp_expire_time = expire_time;
+            /* XXX: parse better, allow hours etc? */
+            break;
 	case OPT_NOTIMESTAMP:
 	    timestamp_format = NULL;
 	    timestamp_specified = true;
@@ -151,6 +190,18 @@ main(int argc, char **argv)
 	    timestamp_format = (optarg != NULL ? optarg : DEFAULT_TIMESTAMP_FORMAT);
 	    timestamp_specified = true;
 	    break;
+	case OPT_PROFILE:
+	    if (strcmp(optarg, "generic") == 0) {
+		/* no operation */
+	    } else if (strcmp(optarg, "mp101") == 0) {
+	        file_types = "mp3,wma";
+	    } else if (strcmp(optarg, "dms1") == 0) {
+	        file_types = "mp3,wma,m4a,wav";
+		expire_time = 1801;
+	    } else {
+		die(_("invalid profile name `%s'\n"), optarg);
+	    }
+	    break;
         case 'b':
 	    background = true;
 	    break;
@@ -175,22 +226,29 @@ main(int argc, char **argv)
         case OPT_HELP:
             printf(_("Usage: %s [OPTIONS]... DIRECTORIES...\n"), argv[0]);
             printf(_("Run the UPnP media server.\n\n"));
-	    printf(_("      --friendly-name=NAME  set display name for media server\n"));
+	    printf(_("      --friendly-name=NAME   set display name for media server\n"));
 #ifdef HAVE_ID3LIB
-	    printf(_("      --disable-id3         do not scan files for ID3 information\n"));
+	    printf(_("      --disable-id3          do not scan files for ID3 information\n"));
 #endif
-	    printf(_("  -v, --verbose[=LEVEL]     set verbosity level (0-4)\n"));
-            printf(_("      --pid-file=FILE       write pid to FILE when up and running\n"));
-            printf(_("  -i, --interface=NAME      listen on a specific interface\n"));
-            printf(_("  -p, --port=PORT           listen on a specific port\n"));
-            printf(_("  -o, --output=LOGFILE      file for logging\n"));
-            printf(_("  -b, --background          go to background (detach)\n"));
-            printf(_("      --no-timestamp        do not prepend timestamp to log entries\n"));
-            printf(_("      --timestamp[=FORMAT]  prepend timestamp with optional time format\n"));
-            printf(_("      --help                display this help and exit\n"));
-            printf(_("      --version             output version information and exit\n"));
+	    printf(_("  -v, --verbose[=LEVEL]      set verbosity level (0-4)\n"));
+            printf(_("      --pid-file=FILE        write pid to FILE when up and running\n"));
+            printf(_("  -i, --interface=NAME       listen on a specific interface\n"));
+            printf(_("  -p, --port=PORT            listen on a specific port\n"));
+            printf(_("  -o, --output=LOGFILE       file for logging\n"));
+            printf(_("  -b, --background           go to background (detach)\n"));
+            printf(_("      --no-timestamp         do not prepend timestamp to log entries\n"));
+            printf(_("      --timestamp[=FORMAT]   prepend timestamp with optional time format\n"));
+	    printf(_("      --profile=NAME         specify profile (see below)\n"));
+            printf(_("      --expire-time=SECONDS  advertisement expire time (default 100)\n"));
+            printf(_("      --help                 display this help and exit\n"));
+            printf(_("      --version              output version information and exit\n"));
 	    printf(_("\nThe directories arguments specify where audio files are located.\n"
 		     "See strftime(3) for description of the timestamp format.\n"));
+	    printf(_("\nA profile specified with --profile enables settings for a certain device.\n"
+		     "The following values are valid for --profile:\n\n"));
+	    printf(_("  generic  no special settings\n"
+	             "  mp101    Netgear MP101\n"
+		     "  dms1     Omnifi DMS1\n"));
             printf(_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
             exit(EXIT_SUCCESS);
         case OPT_VERSION:
@@ -252,7 +310,9 @@ main(int argc, char **argv)
     if (sigemptyset(&signalaction.sa_mask) < 0)
     	die(_("Cannot empty signal set - %s\n"), errstr);
     signalaction.sa_flags = SA_RESTART;
+#ifdef HAVE_STRUCT_SIGACTION_SA_RESTORER
     signalaction.sa_restorer = NULL;
+#endif
     if (sigaction(SIGINT, &signalaction, NULL) < 0)
     	die(_("Cannot register signal handler - %s\n"), errstr);
     if (sigaction(SIGTERM, &signalaction, NULL) < 0)
@@ -280,10 +340,14 @@ main(int argc, char **argv)
 
     while (true) {
     	int sig;
+    	char signame[SIG2STR_MAX];
       	
       	if ((sig = TEMP_FAILURE_RETRY(sigwaitinfo(&signalset, NULL))) < 0)
     	    die(_("sigwaitinfo failed - %s\n"), errstr);
-      	say(2, _("Received signal %d\n"), sig);
+        if (sig2str(sig, signame) == 0)
+      	    say(2, _("Received %s signal\n"), signame);
+      	else
+      	    say(2, _("Received signal %d\n"), sig);
       	if (sig == SIGUSR1) {
       	    /*say(1, "Reloading configuration file...\n");*/ /* XXX: not implemented */
       	}
