@@ -12,17 +12,19 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
 #include <config.h>
 #include <config.h>
+#ifdef HAVE_ID3LIB
 #include <id3.h>                /* id3lib */
 #undef false /* id3lib bug - it shouldn't mess with false and true! */
 #undef true
+#endif
 #include <stdbool.h>            /* Gnulib, C99 */
 #include <stdint.h>		/* Gnulib, C99 */
 #include <dirent.h>		/* POSIX */
@@ -43,18 +45,46 @@
 #include "gmediaserver.h"
 
 #define DEFAULT_ENTRIES_SIZE 512
+#define ROOT_ENTRY_NAME "(root)"
 
 typedef enum {
+    FILE_MP3,
+    /*FILE_MP3_ID3,*/
+    FILE_WMA,
+    FILE_RIFF_WAVE,
+    FILE_M4A,
+    /*FILE_PLS,*/
+    /*FILE_PLAYLIST_M3U,*/
     FILE_UNKNOWN,
-    FILE_MPEG_ADTS,
-    FILE_MP3_ID3,
-    FILE_WMA
 } FileType;
 
+static const char *mime_types[] = {
+    /* FILE_MP3 */		"audio/mpeg",
+    /* FILE_MP3_ID3 */		/*"audio/mpeg",*/
+    /* FILE_WMA */		"audio/x-ms-wma",
+    /* FILE_RIFF_WAVE */	"audio/x-wav",
+    /* FILE_M4A */		"audio/mp4",  
+    /* FILE_PLS */		/*"audio/x-scpls",*/
+    /* FILE_M3U */		/*"audio/m3u",*/
+    /* FILE_UNKNOWN */		"application/octet-stream",
+};
+
+static const char *file_type_names[] = {
+    "mp3",
+    /*"id3mp3",*/
+    "wma",
+    "wav",
+    "m4a",
+    /*"pls",*/
+    /*"m3u",*/
+    "unknown"
+};
+
 static Entry *root_entry;
 static uint32_t entry_count = 0;
 static uint32_t entries_size = 0;
 static Entry **entries;
+char *file_types = "mp3,wma";
 #ifdef HAVE_ID3LIB
 bool id3_enabled = true;
 static ID3Tag *id3;
@@ -147,7 +177,7 @@ add_entry_detail(Entry *entry, EntryDetailType type)
 }
 
 static Entry*
-make_entry(const char *name, int32_t parent, bool directory)
+make_entry(const char *name, int32_t parent, bool directory, uint32_t size)
 {
     Entry *entry;
 
@@ -155,6 +185,8 @@ make_entry(const char *name, int32_t parent, bool directory)
     entry->id = entry_count++;
     entry->parent = parent;
     entry->filename = xstrdup(name);
+    entry->mime_type = "";
+    entry->size = size;
     entry->details = NULL;
 
     if (directory) {
@@ -182,12 +214,33 @@ get_entry_by_id(uint32_t id)
     return entries[id];
 }
 
+
+static bool
+attempt_read_at(int fd, size_t size, off_t pos, uint8_t *buf, const char *fullpath)
+{
+    size_t count;
+
+    if (lseek(fd, pos, SEEK_SET) != pos) {
+	warn(_("%s: cannot seek: %s\n"), fullpath, errstr);
+	close(fd); /* Ignore errors since we opened for reading */
+	return false;
+    }
+
+    count = full_read(fd, buf, size);
+    if (count < 4) {
+	if (errno != 0)
+	    warn(_("%s: cannot read: %s\n"), fullpath, errstr);
+	close(fd); /* Ignore errors since we opened for reading */
+	return false;
+    }
+    return true;
+}
+
 static FileType
 check_file_content_type(const char *fullpath)
 {
     int fd;
-    uint8_t buf[4];
-    size_t count;
+    uint8_t buf[11];
 
     say(4, _("Check content type of file %s\n"), fullpath);
 
@@ -197,30 +250,64 @@ check_file_content_type(const char *fullpath)
 	return FILE_UNKNOWN;
     }
 
-    count = full_read(fd, buf, 4);
-    if (count < 4) {
-	if (errno != 0)
-	    warn(_("%s: cannot read: %s\n"), fullpath, errstr);
-	close(fd); /* Ignore errors since we opened for reading */
+    if (!attempt_read_at(fd, 4, 0, buf, fullpath))
 	return FILE_UNKNOWN;
-    }
 
-    close(fd); /* Ignore errors since we opened for reading */
-    
+    /* MPEG ADTS, layer III, v1 */
     if (buf[0] == 0xFF && (buf[1] & 0xFE) == 0xFA) {
-	say(4, _("Matched type MPEG ADTS for %s\n"), fullpath);
-	return FILE_MPEG_ADTS;
+	say(4, _("Matched type MP3 for %s\n"), fullpath);
+	close(fd); /* Ignore errors since we opened for reading */
+	return FILE_MP3;
     }
+    /* MP3 file with ID3 tag */
     if (buf[0] == 'I' && buf[1] == 'D' && buf[2] == '3') {
 	say(4, _("Matched type MP3 with ID3 tag for %s\n"), fullpath);
-	return FILE_MP3_ID3;
+	close(fd); /* Ignore errors since we opened for reading */
+	return FILE_MP3/*_ID3*/;
     }
+    /* Microsoft ASF */
     if (buf[0] == 0x30 && buf[1] == 0x26 && buf[2] == 0xb2 && buf[3] == 0x75) {
 	say(4, _("Matched type WMA for %s\n"), fullpath);
+	close(fd); /* Ignore errors since we opened for reading */
 	return FILE_WMA;
     }
+    /* RIFF (little-endian), WAVE audio */
+    if (buf[0] == 'R' && buf[1] == 'I' && buf[2] == 'F' && buf[3] == 'F') {
+	if (!attempt_read_at(fd, 4, 8, buf, fullpath))
+	    return FILE_UNKNOWN;
+	if (buf[0] == 'W' && buf[1] == 'A' && buf[2] == 'V' && buf[3] == 'E') {
+	    say(4, _("Matched type RIFF WAVE for %s\n"), fullpath);
+	    close(fd); /* Ignore errors since we opened for reading */
+	    return FILE_RIFF_WAVE;
+	}
+    }
 
+    /* ISO Media, MPEG v4 system, iTunes AAC-LC */
+    if (!attempt_read_at(fd, 4, 4, buf, fullpath))
+	return FILE_UNKNOWN;
+    if (buf[0] == 'f' && buf[1] == 't' && buf[2] == 'y' && buf[3] == 'p') {
+	if (!attempt_read_at(fd, 4, 8, buf, fullpath))
+	    return FILE_UNKNOWN;
+	if (buf[0] == 'M' && buf[1] == '4' && buf[2] == 'A') {
+	    say(4, _("Matched type MPEG v4 iTunes AAC-LC for %s\n"), fullpath);
+	    close(fd); /* Ignore errors since we opened for reading */
+	    return FILE_M4A;
+	}
+    }
+
+#if 0
+    /* Playlist (PLS) */
+    if (!attempt_read_at(fd, 11, 0, buf, fullpath))
+	return FILE_UNKNOWN;
+    if (strncasecmp((char *) buf, "[playlist]", 10) == 0 && (buf[10] == '\r' || buf[10] == '\n')) {
+	say(4, _("Matched PLS playlist for %s\n"), fullpath);
+	close(fd); /* Ignore errors since we opened for reading */
+	return FILE_PLS;
+    }
+#endif
+    
     say(4, _("%s: skipping file - unrecognized content type\n"), fullpath);
+    close(fd); /* Ignore errors since we opened for reading */
     return FILE_UNKNOWN;
 }
 
@@ -275,7 +362,7 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
             return NULL;
         }
 
-        entry = make_entry(name, parent, true);
+        entry = make_entry(name, parent, true, 0);
 	children = xmalloc(sizeof(int32_t) * dirent_count);
 
 	child_count = 0;
@@ -310,14 +397,15 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
             return NULL;
         }
 	type = check_file_content_type(fullpath);
-	if (type == FILE_UNKNOWN)
+	if (!string_in_csv(file_types, file_type_names[type]))
 	    return NULL;
 
 	say(4, _("Adding file %s\n"), fullpath);
-	entry = make_entry(name, parent, false);
+	entry = make_entry(name, parent, false, sb.st_size);
+	entry->mime_type = mime_types[type];
 
 #ifdef HAVE_ID3LIB
-	if (id3_enabled && (type == FILE_MPEG_ADTS || type == FILE_MP3_ID3)) {
+	if (id3_enabled && (type == FILE_MP3/* || type == FILE_MP3_ID3*/)) {
 	    ID3Tag_Clear(id3);
 	    ID3Tag_Link(id3, fullpath);
 
@@ -352,7 +440,7 @@ init_metadata(char **pathv, int pathc)
 	EntryDetail *detail;
 	uint32_t c;
 
-	root_entry = make_entry("", -1, true);
+	root_entry = make_entry(ROOT_ENTRY_NAME, -1, true, 0);
 	children = xmalloc(sizeof(int32_t) * pathc);
 	for (c = 0; c < pathc; c++) {
 	    Entry *entry;
@@ -379,7 +467,7 @@ init_metadata(char **pathv, int pathc)
     } else {
 	EntryDetail *detail;
 	
-	root_entry = scan_entry(pathv[0], "", -1);
+	root_entry = scan_entry(pathv[0], ROOT_ENTRY_NAME, -1);
 	detail = add_entry_detail(root_entry, DETAIL_LOCAL_PATH);
 	detail->data.local_path = xstrdup(pathv[0]);
     }
